;----------------------------------------------------------------------
;
; File: flashpgm.asm
;	Flash program read and write routines. 
;
;	flash_addr	- Sets flash address register to contents of W register
;	flash_write	- Writes contents of flash data register to address in
;		  flash address register
;	flash_read	- Reads flash location pointed to by flash address register
;		  into W
;
; Author:
;	Robert F. Nee
;	robnee@robnee.com
;
; Revisions:
;	last delta 12/15/03 22:12:37
;
;	01/12/02 V1.00
;
;----------------------------------------------------------------------

		cblock 0x120
flash_buffer:64
flash_counter
		endc

flash_pgm_addr	macro addr
		banksel	EEADR
		movlw	LOW addr
		movwf	EEADR
		movlw	HIGH addr
		movwf	EEADRH
		bank00

		endm

;----------------------------------------------------------------------
; Erase a 32 word (64 'bytes') block of program memory.  The address
; of the block to be erased is contained in EEADRH and EEADR.

flash_pgm_erase
		banksel	EECON1
		bsf		EECON1, WREN	; Enable writes
		bsf		EECON1, EEPGD	; Select Program Memory
		bsf		EECON1, FREE	; Enable Row Erase operation

		bcf		INTCON, GIE		; Disable interrupts

		call	flash_write_exec

		bcf		EECON1, WREN	; Disable writes
		bsf		INTCON, GIE		; Enable inturrupts

		bank00

		return

;----------------------------------------------------------------------
; Write contents of flash_buffer to program memory.  The address to be
; written is already stored in EEADRH and EEADR

flash_pgm_write
		banksel	EECON1
		bsf		EECON1, WREN	; Enable writes
		bsf		EECON1, EEPGD	; Select Program Memory
		bcf		EECON1, FREE	; Select Write instead of Erase

		bcf		INTCON, GIE		; Disable interrupts

		; Initialize FSR to the start of the array
		movlw	LOW flash_buffer
		movwf	FSR
		bsf		STATUS, IRP		; flash_buffer is in high bank
		
		banksel flash_counter

		from flash_counter, 32

		 ; Move two bytes to data registers
		 banksel	EEDATA
		 movfw		INDF
		 movwf		EEDATA
		 incf		FSR, f
		 movfw		INDF
		 movwf		EEDATH
		 incf		FSR, f

		 call		flash_write_exec

		 ; Increment program memory address
		 banksel	EEADR
		 incf		EEADR, f

		endfrom

		banksel	EECON1
		bcf		EECON1, WREN	; Disable writes
		bsf		INTCON, GIE		; Enable inturrupts

		bank00
		bcf		STATUS, IRP		; In case FSR is used elsewhere

		return

;----------------------------------------------------------------------
; Require sequence to initiate flash write

flash_write_exec
		; Write initiation sequence
		banksel	EECON1
		movlw	0x55
		movwf	EECON2			; Write 55h
		movlw	0xAA
		movwf	EECON2			; Write AAh

		bsf		EECON1, WR		; Begin erase

		; Dummy instructions that are skipped during write
		nop
		nop

		return
